==> jaysonic-1.2.1.tgz: package/lib/client/ws.js <==
"use strict";

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _wrapNativeSuper(Class) { var _cache = typeof Map === "function" ? new Map() : undefined; _wrapNativeSuper = function _wrapNativeSuper(Class) { if (Class === null || !_isNativeFunction(Class)) return Class; if (typeof Class !== "function") { throw new TypeError("Super expression must either be null or a function"); } if (typeof _cache !== "undefined") { if (_cache.has(Class)) return _cache.get(Class); _cache.set(Class, Wrapper); } function Wrapper() { return _construct(Class, arguments, _getPrototypeOf(this).constructor); } Wrapper.prototype = Object.create(Class.prototype, { constructor: { value: Wrapper, enumerable: false, writable: true, configurable: true } }); return _setPrototypeOf(Wrapper, Class); }; return _wrapNativeSuper(Class); }

function isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Date.prototype.toString.call(Reflect.construct(Date, [], function () {})); return true; } catch (e) { return false; } }

function _construct(Parent, args, Class) { if (isNativeReflectConstruct()) { _construct = Reflect.construct; } else { _construct = function _construct(Parent, args, Class) { var a = [null]; a.push.apply(a, args); var Constructor = Function.bind.apply(Parent, a); var instance = new Constructor(); if (Class) _setPrototypeOf(instance, Class.prototype); return instance; }; } return _construct.apply(null, arguments); }

function _isNativeFunction(fn) { return Function.toString.call(fn).indexOf("[native code]") !== -1; }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

var _ = require("lodash");

var _require = require("../functions"),
    formatRequest = _require.formatRequest;

var _require2 = require("../constants"),
    ERR_CODES = _require2.ERR_CODES,
    ERR_MSGS = _require2.ERR_MSGS;

var WSClient =
/*#__PURE__*/
function (_EventTarget) {
  _inherits(WSClient, _EventTarget);

  function WSClient(options) {
    var _this;

    _classCallCheck(this, WSClient);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(WSClient).call(this));

    if (!(_assertThisInitialized(_this) instanceof WSClient)) {
      return _possibleConstructorReturn(_this, new WSClient(options));
    }

    var defaults = {
      url: "ws://127.0.0.1:8100",
      version: "2.0",
      delimiter: "\n",
      timeout: 30,
      retries: 2
    };
    _this.message_id = 1;
    _this.serving_message_id = 1;
    _this.pendingCalls = {};
    _this.pendingBatches = {};
    _this.attached = false;
    /**
     * we can receive whole messages, or parital so we need to buffer
     *
     * whole message: {"jsonrpc": 2.0, "params": ["hello"], id: 1}
     *
     * partial message: {"jsonrpc": 2.0, "params"
     */

    _this.messageBuffer = "";
    _this.responseQueue = {};
    _this.options = _.merge(defaults, options || {});
    _this.options.timeout = _this.options.timeout * 1000;
    var retries = _this.options.retries;
    _this.remainingRetries = retries;
    return _this;
  }

  _createClass(WSClient, [{
    key: "connect",
    value: function connect() {
      var _this2 = this;

      return new Promise(function (resolve, reject) {
        var _this2$options = _this2.options,
            url = _this2$options.url,
            protocols = _this2$options.protocols;
        _this2.client = new window.WebSocket(url, protocols);

        _this2.close();

        _this2.listen();

        _this2.client.onopen = function (event) {
          resolve(event);
        };

        _this2.client.onerror = function (error) {
          reject(error);
        };
      });
    }
  }, {
    key: "close",
    value: function close() {
      var _this3 = this;

      this.client.onclose = function () {
        if (_this3.remainingRetries) {
          _this3.remainingRetries -= 1; // console.log(
          //   `Connection failed. ${this.remainingRetries} attempts left.`
          // );

          setTimeout(function () {
            _this3.initClient();
          }, _this3.options.timeout);
        }
      };
    }
  }, {
    key: "request",
    value: function request() {
      var _this4 = this;

      return {
        message: function message(method, params) {
          var request = formatRequest({
            method: method,
            params: params,
            id: _this4.message_id,
            options: _this4.options
          });
          _this4.message_id += 1;
          return request;
        },
        send: function send(method, params) {
          return new Promise(function (resolve, reject) {
            var requestId = _this4.message_id;
            _this4.pendingCalls[requestId] = {
              resolve: resolve,
              reject: reject
            };

            _this4.client.send(_this4.request().message(method, params));

            setTimeout(function () {
              if (_this4.pendingCalls[requestId]) {
                var error = _this4.sendError({
                  id: requestId,
                  code: ERR_CODES.timeout,
                  message: ERR_MSGS.timeout
                });

                delete _this4.pendingCalls[requestId];
                reject(error);
              }
            }, _this4.options.timeout);
          });
        },
        notify: function notify(method, params) {
          var request = formatRequest({
            method: method,
            params: params,
            options: _this4.options
          });
          return new Promise(function (resolve, reject) {
            _this4.client.send(request);

            resolve("notification sent");

            _this4.client.onerror = function (error) {
              reject(error);
            };
          });
        }
      };
    }
  }, {
    key: "verifyData",
    value: function verifyData(chunk) {
      try {
        // sometimes split messages have empty string at the end
        // just ignore it
        if (chunk !== "") {
          // will throw an error if not valid json
          var message = JSON.parse(chunk);

          if (_.isArray(message)) {
            // possible batch request
            this.handleBatchResponse(message);
          } else if (!_.isObject(message)) {
            // error out if it cant be parsed
            var error = this.sendError({
              id: null,
              code: ERR_CODES.parseError,
              message: ERR_MSGS.parseError
            });
            this.handleError(error);
          } else if (!message.id) {
            // no id, so assume notification
            this.handleNotification(message);
          } else if (message.error) {
            // got an error back so reject the message
            var _error = this.sendError({
              jsonrpc: message.jsonrpc,
              id: message.id,
              code: message.error.code,
              message: message.error.message
            });

            this.handleError(_error);
          } else if (!message.method) {
            // no method, so assume response
            this.serving_message_id = message.id;
            this.responseQueue[this.serving_message_id] = message;
            this.handleResponse(message);
          } else {
            throw new Error();
          }
        }
      } catch (e) {
        if (e instanceof TypeError) {
          var _error2 = this.sendError({
            id: this.serving_message_id,
            code: ERR_CODES.parseError,
            message: ERR_MSGS.parseError
          });

          this.handleError(_error2);
        } else {
          var _error3 = this.sendError({
            id: this.serving_message_id,
            code: ERR_CODES.internal,
            message: ERR_MSGS.internal
          });

          this.handleError(_error3);
        }
      }
    }
  }, {
    key: "batch",
    value: function batch(requests) {
      var _this5 = this;

      /**
       * should receive a list of request objects
       * [client.request.message(), client.request.message()]
       * send a single request with that, server should handle it
       *
       * We want to store the IDs for the requests in the batch in an array
       * Use this to reference the batch response
       * The spec has no explaination on how to do that, so this is the solution
       */
      return new Promise(function (resolve, reject) {
        var batchIds = [];
        var batchRequests = [];
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = requests[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var _request = _step.value;
            var json = JSON.parse(_request);
            batchRequests.push(json);

            if (json.id) {
              batchIds.push(json.id);
            }
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator["return"] != null) {
              _iterator["return"]();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }

        _this5.pendingBatches[String(batchIds)] = {
          resolve: resolve,
          reject: reject
        };
        var request = JSON.stringify(batchRequests);

        try {
          _this5.client.send(request + _this5.options.delimiter);
        } catch (e) {
          if (e instanceof TypeError) {
            // this.client is probably undefined
            reject(new Error("Unable to send request. ".concat(e.message)));
          }
        }

        setTimeout(function () {
          if (_this5.pendingBatches[String(batchIds)]) {
            var error = _this5.sendError({
              id: null,
              code: ERR_CODES.timeout,
              message: ERR_MSGS.timeout
            });

            delete _this5.pendingBatches[String(batchIds)];
            reject(error);
          }
        }, _this5.options.timeout);
      });
    }
  }, {
    key: "handleBatchResponse",
    value: function handleBatchResponse(batch) {
      var _this6 = this;

      var batchResponseIds = [];
      batch.forEach(function (message) {
        if (message.id) {
          batchResponseIds.push(message.id);
        }
      });

      var _loop = function _loop() {
        var ids = _Object$keys[_i];

        if (_.isEmpty(_.difference(JSON.parse("[".concat(ids, "]")), batchResponseIds))) {
          batch.forEach(function (message) {
            if (message.error) {
              // reject the whole message if there are any errors
              _this6.pendingBatches[ids].reject(batch);
            }
          });

          _this6.pendingBatches[ids].resolve(batch);
        }
      };

      for (var _i = 0, _Object$keys = Object.keys(this.pendingBatches); _i < _Object$keys.length; _i++) {
        _loop();
      }
    }
  }, {
    key: "handleNotification",
    value: function handleNotification(message) {
      this.dispatchEvent(new CustomEvent("notify", {
        detail: message
      }));
    }
    /**
     * @params {String} [method] method to subscribe to
     * @params {Function} [cb] callback function to invoke on notify
     */

  }, {
    key: "subscribe",
    value: function subscribe(method, cb) {
      this.addEventListener("notify", function (_ref) {
        var detail = _ref.detail;

        try {
          if (detail.method === method) {
            return cb(null, detail);
          }
        } catch (e) {
          return cb(e);
        }
      });
    }
  }, {
    key: "listen",
    value: function listen() {
      var _this7 = this;

      this.client.onmessage = function (message) {
        _this7.verifyData(message.data);
      };
    }
  }, {
    key: "handleResponse",
    value: function handleResponse(message) {
      if (!(this.pendingCalls[message.id] === undefined)) {
        var response = this.responseQueue[message.id];
        this.pendingCalls[message.id].resolve(response);
        delete this.responseQueue[message.id];
      }
    }
  }, {
    key: "handleError",
    value: function handleError(error) {
      var response = error;
      this.pendingCalls[error.id].reject(response);
    }
  }, {
    key: "sendError",
    value: function sendError(_ref2) {
      var jsonrpc = _ref2.jsonrpc,
          id = _ref2.id,
          code = _ref2.code,
          message = _ref2.message;
      var response;

      if (this.options.version === "2.0") {
        response = {
          jsonrpc: jsonrpc || this.options.version,
          error: {
            code: code,
            message: message || "Unknown Error"
          },
          id: id
        };
      } else {
        response = {
          result: null,
          error: {
            code: code,
            message: message || "Unknown Error"
          },
          id: id
        };
      }

      return response;
    }
  }]);

  return WSClient;
}(_wrapNativeSuper(EventTarget));

module.exports = WSClient;

==> jaysonic-1.3.0.tgz: package/lib/client/ws.js <==
"use strict";

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

var _ = require("lodash");

var WebSocket = require("ws");

var Client = require(".");

var _require = require("../functions"),
    formatRequest = _require.formatRequest;

var _require2 = require("../constants"),
    ERR_CODES = _require2.ERR_CODES,
    ERR_MSGS = _require2.ERR_MSGS;

var WSClient =
/*#__PURE__*/
function (_Client) {
  _inherits(WSClient, _Client);

  function WSClient(options) {
    var _this;

    _classCallCheck(this, WSClient);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(WSClient).call(this));

    if (!(_assertThisInitialized(_this) instanceof WSClient)) {
      return _possibleConstructorReturn(_this, new WSClient(options));
    }

    var defaults = {
      url: "ws://127.0.0.1:8100",
      version: "2.0",
      delimiter: "\n",
      timeout: 30,
      retries: 2
    };
    _this.message_id = 1;
    _this.serving_message_id = 1;
    _this.pendingCalls = {};
    _this.pendingBatches = {};
    _this.attached = false;
    /**
     * we can receive whole messages, or parital so we need to buffer
     *
     * whole message: {"jsonrpc": 2.0, "params": ["hello"], id: 1}
     *
     * partial message: {"jsonrpc": 2.0, "params"
     */

    _this.messageBuffer = "";
    _this.responseQueue = {};
    _this.options = _.merge(defaults, options || {});
    _this.options.timeout = _this.options.timeout * 1000;
    var retries = _this.options.retries;
    _this.remainingRetries = retries;
    return _this;
  }

  _createClass(WSClient, [{
    key: "connect",
    value: function connect() {
      var _this2 = this;

      return new Promise(function (resolve, reject) {
        var _this2$options = _this2.options,
            url = _this2$options.url,
            perMessageDeflate = _this2$options.perMessageDeflate;
        _this2.client = new WebSocket(url, perMessageDeflate);

        _this2.close();

        _this2.listen();

        _this2.client.onopen = function (event) {
          resolve(event);
        };

        _this2.client.onerror = function (error) {
          reject(error);
        };
      });
    }
  }, {
    key: "close",
    value: function close() {
      var _this3 = this;

      this.client.onclose = function () {
        if (_this3.remainingRetries) {
          _this3.remainingRetries -= 1;
          process.stdout.write("Connection failed. ".concat(_this3.remainingRetries, " attempts left.\n"));
          setTimeout(function () {
            _this3.connect()["catch"](function () {});
          }, _this3.options.timeout);
        }
      };
    }
  }, {
    key: "request",
    value: function request() {
      var _this4 = this;

      return {
        message: function message(method, params) {
          var request = formatRequest({
            method: method,
            params: params,
            id: _this4.message_id,
            options: _this4.options
          });
          _this4.message_id += 1;
          return request;
        },
        send: function send(method, params) {
          return new Promise(function (resolve, reject) {
            var requestId = _this4.message_id;
            _this4.pendingCalls[requestId] = {
              resolve: resolve,
              reject: reject
            };

            _this4.client.send(_this4.request().message(method, params));

            setTimeout(function () {
              if (_this4.pendingCalls[requestId]) {
                var error = _this4.sendError({
                  id: requestId,
                  code: ERR_CODES.timeout,
                  message: ERR_MSGS.timeout
                });

                delete _this4.pendingCalls[requestId];
                reject(error);
              }
            }, _this4.options.timeout);
          });
        },
        notify: function notify(method, params) {
          var request = formatRequest({
            method: method,
            params: params,
            options: _this4.options
          });
          return new Promise(function (resolve, reject) {
            _this4.client.send(request);

            resolve("notification sent");

            _this4.client.onerror = function (error) {
              reject(error);
            };
          });
        }
      };
    }
  }, {
    key: "verifyData",
    value: function verifyData(chunk) {
      try {
        // sometimes split messages have empty string at the end
        // just ignore it
        if (chunk !== "") {
          // will throw an error if not valid json
          var message = JSON.parse(chunk);

          if (_.isArray(message)) {
            // possible batch request
            this.handleBatchResponse(message);
          } else if (!_.isObject(message)) {
            // error out if it cant be parsed
            var error = this.sendError({
              id: null,
              code: ERR_CODES.parseError,
              message: ERR_MSGS.parseError
            });
            this.handleError(error);
          } else if (!message.id) {
            // no id, so assume notification
            this.handleNotification(message);
          } else if (message.error) {
            // got an error back so reject the message
            var _error = this.sendError({
              jsonrpc: message.jsonrpc,
              id: message.id,
              code: message.error.code,
              message: message.error.message
            });

            this.handleError(_error);
          } else if (!message.method) {
            // no method, so assume response
            this.serving_message_id = message.id;
            this.responseQueue[this.serving_message_id] = message;
            this.handleResponse(message);
          } else {
            throw new Error();
          }
        }
      } catch (e) {
        if (e instanceof SyntaxError) {
          var _error2 = this.sendError({
            id: this.serving_message_id,
            code: ERR_CODES.parseError,
            message: "Unable to parse message: '".concat(chunk, "'")
          });

          this.handleError(_error2);
        } else {
          var _error3 = this.sendError({
            id: this.serving_message_id,
            code: ERR_CODES.internal,
            message: "Unable to parse message: '".concat(chunk, "'")
          });

          this.handleError(_error3);
        }
      }
    }
  }, {
    key: "batch",
    value: function batch(requests) {
      var _this5 = this;

      /**
       * should receive a list of request objects
       * [client.request.message(), client.request.message()]
       * send a single request with that, server should handle it
       *
       * We want to store the IDs for the requests in the batch in an array
       * Use this to reference the batch response
       * The spec has no explaination on how to do that, so this is the solution
       */
      return new Promise(function (resolve, reject) {
        var batchIds = [];
        var batchRequests = [];
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = requests[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var _request = _step.value;
            var json = JSON.parse(_request);
            batchRequests.push(json);

            if (json.id) {
              batchIds.push(json.id);
            }
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator["return"] != null) {
              _iterator["return"]();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }

        _this5.pendingBatches[String(batchIds)] = {
          resolve: resolve,
          reject: reject
        };
        var request = JSON.stringify(batchRequests);

        try {
          _this5.client.send(request + _this5.options.delimiter);
        } catch (e) {
          if (e instanceof TypeError) {
            // this.client is probably undefined
            reject(new Error("Unable to send request. ".concat(e.message)));
          }
        }

        setTimeout(function () {
          if (_this5.pendingBatches[String(batchIds)]) {
            var error = _this5.sendError({
              id: null,
              code: ERR_CODES.timeout,
              message: ERR_MSGS.timeout
            });

            delete _this5.pendingBatches[String(batchIds)];
            reject(error);
          }
        }, _this5.options.timeout);
      });
    }
  }, {
    key: "handleBatchResponse",
    value: function handleBatchResponse(batch) {
      var _this6 = this;

      var batchResponseIds = [];
      batch.forEach(function (message) {
        if (message.id) {
          batchResponseIds.push(message.id);
        }
      });

      var _loop = function _loop() {
        var ids = _Object$keys[_i];

        if (_.isEmpty(_.difference(JSON.parse("[".concat(ids, "]")), batchResponseIds))) {
          batch.forEach(function (message) {
            if (message.error) {
              // reject the whole message if there are any errors
              if (_this6.pendingBatches[ids] !== undefined) {
                _this6.pendingBatches[ids].reject(batch);

                delete _this6.pendingBatches[ids];
              }
            }
          });

          if (_this6.pendingBatches[ids] !== undefined) {
            _this6.pendingBatches[ids].resolve(batch);

            delete _this6.pendingBatches[ids];
          }
        }
      };

      for (var _i = 0, _Object$keys = Object.keys(this.pendingBatches); _i < _Object$keys.length; _i++) {
        _loop();
      }
    }
  }, {
    key: "handleNotification",
    value: function handleNotification(message) {
      this.emit("notify", {
        detail: message
      });
    }
    /**
     * @params {String} [method] method to subscribe to
     * @params {Function} [cb] callback function to invoke on notify
     */

  }, {
    key: "subscribe",
    value: function subscribe(method, cb) {
      this.on("notify", function (_ref) {
        var detail = _ref.detail;

        try {
          if (detail.method === method) {
            return cb(null, detail);
          }
        } catch (e) {
          return cb(e);
        }
      });
    }
  }, {
    key: "listen",
    value: function listen() {
      var _this7 = this;

      this.client.onmessage = function (message) {
        _this7.verifyData(message.data);
      };
    }
  }, {
    key: "handleResponse",
    value: function handleResponse(message) {
      if (!(this.pendingCalls[message.id] === undefined)) {
        var response = this.responseQueue[message.id];
        this.pendingCalls[message.id].resolve(response);
        delete this.responseQueue[message.id];
      }
    }
  }, {
    key: "handleError",
    value: function handleError(error) {
      var response = error;

      try {
        this.pendingCalls[error.id].reject(response);
      } catch (e) {
        if (e instanceof TypeError) {
          // probably a parse error, which might not have an id
          process.stdout.write("Message has no outstanding calls: ".concat(JSON.stringify(error), "\n"));
        }
      }
    }
  }, {
    key: "sendError",
    value: function sendError(_ref2) {
      var jsonrpc = _ref2.jsonrpc,
          id = _ref2.id,
          code = _ref2.code,
          message = _ref2.message;
      var response;

      if (this.options.version === "2.0") {
        response = {
          jsonrpc: jsonrpc || this.options.version,
          error: {
            code: code,
            message: message || "Unknown Error"
          },
          id: id
        };
      } else {
        response = {
          result: null,
          error: {
            code: code,
            message: message || "Unknown Error"
          },
          id: id
        };
      }

      return response;
    }
  }]);

  return WSClient;
}(Client);

module.exports = WSClient;
